% Comment
%
%  A class to estimate parameters for a mixture of Von Mises distribution
%
% 
%
% 
%
% Data members
%  
% Methods
%  
% History
%  Sept 14, 2011 created by Chanwoo Kim
%
% Todo
%  Testing the code on the RM1 database
% 
% Example
%
%  vonMises = CMixtureVonMises(4, 100, 15 / 180 * pi, 15 / 180 * pi)
%  vonMises = vonMises.Init();
%  [vonMises, iResult] = vonMises.EstimateMixtureParm(ad_angle, ad_rho)
%  [d_c_0, d_c_1, d_mu_0, d_mu_1, d_k_0, d_k_1]  = vonMises.GetModel();

classdef CMixtureVonMises
    properties (SetAccess = private)
        
        %
        % Parameters for the mixture
        %
        m_d_c_0;
        m_d_c_1;
        m_d_mu_0;
        m_d_mu_1;
        m_d_k_0;
        m_d_k_1;
        
        %
        % Internal variables
        %
        m_iIndex;
        m_iNumIter;
        m_d_k_1_init;
        m_d_theta_0;
        m_d_theta_1;
        m_d_mu_1_sm;
        m_d_mu_0_sm;
        m_d_k_1_sm;
        m_d_k_0_sm;
    end
    
    methods
        function [obj] = CMixtureVonMises(iNumIter, d_k_1_init, d_theta_0, d_theta_1)
            obj.m_iNumIter   = iNumIter;
            obj.m_d_k_1_init = d_k_1_init;
            obj.m_d_theta_0  = d_theta_0;
            obj.m_d_theta_1  = d_theta_1;
           
            obj.m_iIndex     = 0; 
            obj.m_d_mu_1_sm  = 0;
            
            obj.m_d_c_0  = 1;
            obj.m_d_c_1  = 0;
            obj.m_d_mu_0 = 0;
            obj.m_d_mu_1 = 0;
            obj.m_d_k_0  = 0;
            obj.m_d_k_1  = 0;
            obj.m_d_k_1_sm = 0;
        end
        
        function [obj] = Init(obj)
            obj.m_iIndex = 0; 
        end
        
        function [obj, iResult]  = EstimateMixtureParm(obj, ad_angle, ad_rho)
            
            iIndex     = obj.m_iIndex;
            iNumIter   = obj.m_iNumIter;
            d_k_1_init = obj.m_d_k_1_init;
            d_theta_0  = obj.m_d_theta_0;
            d_theta_1  = obj.m_d_theta_1;
            d_mu_1_sm  = obj.m_d_mu_1_sm;
            d_k_1_sm   = obj.m_d_k_1_sm;
            d_mu_0_sm  = obj.m_d_mu_0_sm;
            d_k_0_sm   = obj.m_d_k_0_sm;
            

            ad_K_0 = find(abs(ad_angle) >= d_theta_0);
            ad_K_1 = find(abs(ad_angle) <  d_theta_0);

            iResult = 1;
            
            
            
            if (length(ad_K_0) > 0 & length(ad_K_1) > 0)
                
      
                
                ad_I_0 = zeros(length(ad_angle), 1);
                ad_I_0(ad_K_0) = 1;

                ad_I_1 = zeros(length(ad_angle), 1);
                ad_I_1(ad_K_1) = 1;

                ad_z  = exp(j * 2 * ad_angle');

                d_mean_z_0 = sum(ad_I_0 .* ad_rho .* ad_z) / max(sum(ad_I_0 .* ad_rho ), eps);
                d_mean_z_1 = sum(ad_I_1 .* ad_rho .* ad_z) / max(sum(ad_I_1 .* ad_rho ), eps);

                d_c_0 = sum(ad_rho(ad_K_0))/  sum(ad_rho); 
                d_c_1 = 1 - d_c_0;

                d_mu_0 = angle(d_mean_z_0);
                d_mu_1 = angle(d_mean_z_1);


                if (d_mu_1 <=  - 2 * d_theta_0)
                    d_mu_1 = -2 * d_theta_0;
                end
                if (d_mu_1 >= 2 * d_theta_0)
                    d_mu_1 = 2 * d_theta_0;
                end

                if (d_mu_1 - 2 * d_theta_1 <= d_mu_0 & d_mu_0 <= d_mu_1)
                    d_mu_0 = d_mu_1 - 2 * d_theta_1;
                end
                if (d_mu_1 + 2 * d_theta_1 >= d_mu_0 & d_mu_0 >= d_mu_1)
                    d_mu_0 = d_mu_1 + 2 * d_theta_1;
                end


                d_k_0  = EstimKappa3(abs(d_mean_z_0));
                d_k_1  = EstimKappa3(abs(d_mean_z_1));
                
                
               if iIndex == 0
                      d_mu_1_sm = 0; %d_mu_1;
                      d_k_1_sm  = d_k_1_init;
                      d_mu_0_sm = d_mu_0;
                      d_k_0_sm  = d_k_0; %_init;
               end
               
               
               
                %
                % Updating the parameter using the EM algorithm
                %
                for t = 1 :  iNumIter
                    
                   
            
                    
                    M_pdf_0 =  exp(d_k_0 * cos(2 * ad_angle - d_mu_0)) / (pi * besseli(0, d_k_0));
                    M_pdf_1 =  exp(d_k_1 * cos(2 * ad_angle - d_mu_1)) / (pi * besseli(0, d_k_1));
                    M_d_T_0 = (d_c_0 * M_pdf_0) ./ max((d_c_0 * M_pdf_0 + d_c_1 * M_pdf_1), eps);
                    M_d_T_0 = M_d_T_0(:);
                    M_d_T_1 = (d_c_1 * M_pdf_1) ./ max((d_c_0 * M_pdf_0 + d_c_1 * M_pdf_1), eps);
                    M_d_T_1 = M_d_T_1(:);

                    d_mean_z_0 = sum(ad_rho .* M_d_T_0 .* ad_z) / max(sum(M_d_T_0 .* ad_rho ), eps);
                    d_mean_z_1 = sum(ad_rho .* M_d_T_1 .* ad_z) / max(sum(M_d_T_1 .* ad_rho ), eps);

                    d_c_0 = sum(ad_rho .* M_d_T_0) / sum(ad_rho);
                    d_c_1 = 1 - d_c_0; 

                    d_mu_0 = angle(d_mean_z_0);
                    d_mu_1 = angle(d_mean_z_1);

                    if (d_mu_1 <= -2 * d_theta_0)
                        d_mu_1  = -2 * d_theta_0;
                    end
                    if (d_mu_1 >= 2 * d_theta_0)
                        d_mu_1  = 2 * d_theta_0;
                    end

                    if (d_mu_1 - 2 * d_theta_1 <= d_mu_0 & d_mu_0 <= d_mu_1)
                        d_mu_0 = d_mu_1 - 2 * d_theta_1;
                    end
                    if (d_mu_1 + 2 * d_theta_1 >= d_mu_0 & d_mu_0 >= d_mu_1)
                        d_mu_0 = d_mu_1 + 2 * d_theta_1;
                    end

                    d_k_0  = EstimKappa3(abs(d_mean_z_0));
                    d_k_1  = EstimKappa3(abs(d_mean_z_1));
                    
                     d_mu_1 = 0.95 * d_mu_1_sm + 0.05 * d_mu_1;
                     d_k_1  = 0.95 * d_k_1_sm  + 0.05 * d_k_1; 
                %    d_k_1  = 0.98 * d_k_1_sm  + 0.02 * d_k_1; 
                    
                    
                    
%                    d_mu_0 = 0.9 * d_mu_0_sm + 0.1 * d_mu_0;
%                    d_k_0  = 0.9 * d_k_0_sm  + 0.1 * d_k_0; 
                 %   d_mu_1    = d_mu_1_sm;
                 %   d_k_1     = d_k_1_sm; %d_k_1_init;
                end
                
                [d_mu_1, d_k_1]
                
                d_mu_1_sm = d_mu_1; %0.95 * d_mu_1_sm + 0.05 * d_mu_1;
                d_k_1_sm  = d_k_1; %0.95 * d_k_1_sm  + 0.05 * d_k_1; 
                d_mu_0_sm = d_mu_0;
                d_k_0_sm  = d_k_0;
                
                obj.m_d_c_0    = d_c_0;
                obj.m_d_c_1    = d_c_1;
                obj.m_d_mu_0   = d_mu_0;
                obj.m_d_mu_1   = d_mu_1;
                obj.m_d_k_0    = d_k_0;
                obj.m_d_k_1    = d_k_1;
                obj.m_d_k_1_sm = d_k_1_sm;
                obj.m_d_k_0_sm = d_k_0_sm;
                obj.m_d_mu_1_sm = d_mu_1_sm;
                obj.m_d_mu_0_sm = d_mu_0_sm;
                
            elseif isempty(ad_K_0) 
                        iResult = 2;
            elseif isempty(ad_K_1)
                        iResult = 3;
            end
            
      
            
            iIndex = iIndex + 1;
            
            obj.m_iIndex    = iIndex;
            
            
        end
        
         function [d_c_0, d_c_1, d_mu_0, d_mu_1, d_k_0, d_k_1]  = GetModel(obj)
            d_c_0  = obj.m_d_c_0;
            d_c_1  = obj.m_d_c_1;
            d_mu_0 = obj.m_d_mu_0;
            d_mu_1 = obj.m_d_mu_1;
            d_k_0  = obj.m_d_k_0;
            d_k_1  = obj.m_d_k_1;
         end
        
    end 
end
